@@ -1,5 +1,5 @@
 #ifndef LINT
-static char *rcsid="$Id: smscanner.c 323 2005-05-18 21:51:42Z crosser $";
+static char *rcsid="$Id: smscanner.c 434 2006-12-17 10:50:57Z crosser $";
 #endif
                                                                                 
 /*
@@ -38,12 +38,20 @@ static char *rcsid="$Id: smscanner.c 323 2005-05-18 21:51:42Z crosser $";
 # include <fcntl.h>
 #endif
 #include <signal.h>
-#include <sys/socket.h>
-#include <arpa/inet.h>
-#include <netinet/in.h>
-#include <sys/un.h>
+#ifdef HAVE_SYS_SOCKET_H
+# include <sys/socket.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
+# include <arpa/inet.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
+# include <netinet/in.h>
+#endif
+#ifdef HAVE_SYS_UN_H
+# include <sys/un.h>
+#endif
 #ifndef OPTARG_DEFINED
-#include <getopt.h>
+# include <getopt.h>
 #endif
 
 #include <libmilter/mfapi.h>
@@ -72,6 +80,49 @@ terminate(int sig)
 }
 */
 
+static char *conn_names[] = {
+	"daemon_name",
+	"if_name",
+	"if_addr",
+	"j",
+	"_",
+	NULL
+};
+
+static char *helo_names[] = {
+	"tls_version",
+	"cipher",
+	"cipher_bits",
+	"cert_subject",
+	"cert_issuer",
+	NULL
+};
+
+static char *envfrom_names[] = {
+	"i",
+	"auth_type",
+	"auth_authen",
+	"auth_ssf",
+	"auth_author",
+	"mail_mailer",
+	"mail_host",
+	"mail_addr",
+	NULL
+};
+
+static char *envrcpt_names[] = {
+	"rcpt_mailer",
+	"rcpt_host",
+	"rcpt_addr",
+	NULL
+};
+
+typedef struct _svmacro {
+	struct _svmacro *next;
+	char *name;
+	char *value;
+} svmacro_t;
+
 /* Header scanners presume that text of the headers stay available for
    the whole time of message processing.  Content-Type, boundary, e.t.c.
    in the mattr structure point into the middle of the headers.
@@ -93,6 +144,8 @@ typedef struct _mlfiPriv
 		int msgcount;
 		char *peerstr;
 		char *helostr;
+		char *myfqdn;
+		svmacro_t *svmacros;
 	} c;
 
 	struct _m {
@@ -113,6 +166,8 @@ mlfi_priv_init_c(mlfiPriv_t *priv)
 	priv->c.msgcount=0;
 	priv->c.peerstr=NULL;
 	priv->c.helostr=NULL;
+	priv->c.myfqdn=NULL;
+	priv->c.svmacros=NULL;
 	return 0;
 }
 
@@ -120,21 +175,38 @@ mlfi_priv_init_c(mlfiPriv_t *priv)
 void
 mlfi_priv_cleanup_c(mlfiPriv_t *priv)
 {
+	svmacro_t *tmp,*next;
+
 	if (priv->c.peerstr) free(priv->c.peerstr);
 	priv->c.peerstr=NULL;
 	if (priv->c.helostr) free(priv->c.helostr);
 	priv->c.helostr=NULL;
+	if (priv->c.myfqdn) free(priv->c.myfqdn);
+	priv->c.myfqdn=NULL;
+	for (tmp=priv->c.svmacros;tmp;tmp=next) {
+		next=tmp->next;
+		if (tmp->name) free(tmp->name);
+		if (tmp->value) free(tmp->value);
+		free(tmp);
+	}
+	priv->c.svmacros=NULL;
 }
 
 /* initialize message part of priv strucure */
 int
 mlfi_priv_init_m(mlfiPriv_t *priv)
 {
+	svmacro_t *saved;
+
 	if ((priv->m.vp=vp_create_vp()) == NULL) {
 		ERRLOG((LOG_ERR,"vp_create_vp failed: %m"));
 		return -1;
 	}
 	/* DPRINT(("mlfi_priv_init_m: varpool at %p\n",priv->vp)); */
+	for (saved=priv->c.svmacros;saved;saved=saved->next) {
+		vp_set_str(priv->m.vp,saved->name,saved->value);
+	}
+	if (priv->c.myfqdn) vp_set_str(priv->m.vp,"myfqdn",priv->c.myfqdn);
 	vp_set_str(priv->m.vp,VP_ANSWER_STR,"0 250 Acceptable");
 	priv->m.firsthdr=NULL;
 	priv->m.lasthdr=(hdr_t*)&(priv->m.firsthdr);
@@ -256,9 +328,34 @@ mlfi_termmode(SMFICTX *ctx,int stopcode)
 	else if (p) msg=p+3;
 	else msg=NULL;
 	if (msg) while (*msg == ' ') msg++;
-	DPRINT(("mlfi_termmode: rc=%d rcode=\"%s\" xcode=\"%s\"\n",
-			rc,rcode,xcode?xcode:"(nil)"));
-	(void)smfi_setreply(ctx,rcode,xcode[0]?xcode:NULL,msg);
+#if 0
+	ERRLOG((LOG_INFO,"mlfi_termmode: rc=%d rcode=\"%s\" xcode=\"%s\" msg=\"%s\"\n",
+			rc,rcode,xcode?xcode:"<null>",msg?msg:"<null>"));
+#else
+	DPRINT(("mlfi_termmode: rc=%d rcode=\"%s\" xcode=\"%s\" msg=\"%s\"\n",
+			rc,rcode,xcode?xcode:"<null>",msg?msg:"<null>"));
+#endif
+	if (rc) {
+		char *emsg,*p;
+		int pcount=0;
+
+		for (p=msg;*p;p++) if (*p == '%') pcount++;
+		emsg=malloc(strlen(msg)+pcount+1);
+		if (msg) {
+			q=emsg;
+			for (p=msg;*p;p++) {
+				if (*p == '%') *q++='%';
+				*q++=*p;
+			}
+			*q='\0';
+		}
+		/* emsg==NULL is allowed in smfi_setreply */
+		if (smfi_setreply(ctx,rcode,xcode[0]?xcode:NULL,emsg)) {
+			ERRLOG((LOG_ERR,"smfi_setreply(ctx,%s,%s,%s) error",
+				rcode,xcode[0]?xcode:"<null>",emsg));
+		}
+		if (emsg) free(emsg);
+	}
 
 	switch (rc) {
 	case 0:
@@ -282,6 +379,41 @@ mlfi_termmode(SMFICTX *ctx,int stopcode)
 	}
 }
 
+static void
+mlfi_macros2vp(SMFICTX *ctx,char *names[],varpool_t vp,svmacro_t **svmacro)
+{
+	char *mn,*mv;
+	char vname[32],mname[32];
+	int i;
+
+	for (i=0;names[i];i++) {
+		mn=names[i];
+		snprintf(mname,sizeof(mname),"{%s}",mn);
+		mv=smfi_getsymval(ctx,mname);
+		if (!mv) continue;
+		DPRINT(("sendmail env %s=%s\n",mn,mv));
+		snprintf(vname,sizeof(vname),"smfi_%s",mn);
+		vp_set_str(vp,vname,mv);
+		if (svmacro) {
+			svmacro_t *tmp=(svmacro_t*)malloc(sizeof(svmacro_t));
+
+			/* malloc paranoia... */
+			if (tmp && ((tmp->name=(char*)malloc(strlen(mn)+1))))
+				strcpy(tmp->name,mn);
+			if (tmp && ((tmp->value=(char*)malloc(strlen(mv)+1))))
+				strcpy(tmp->value,mv);
+			if (tmp && tmp->name && tmp->value) {
+				tmp->next=*svmacro;
+				(*svmacro)=tmp;
+			} else {
+				if (tmp && tmp->name) free(tmp->name);
+				if (tmp && tmp->value) free(tmp->value);
+				if (tmp) free(tmp);
+			}
+		}
+	}
+}
+
 sfsistat
 mlfi_conn(SMFICTX *ctx,char *hostname,_SOCK_ADDR *hostaddr)
 {
@@ -291,6 +423,7 @@ mlfi_conn(SMFICTX *ctx,char *hostname,_SOCK_ADDR *hostaddr)
 	char caddr[64];
 	int port;
 	mlfiPriv_t *priv = mlfi_priv(ctx);
+	char *myfqdn;
 
 	DPRINT(("mlfi_conn: %s\n",hostname));
 	if (priv == NULL) return SMFIS_TEMPFAIL;
@@ -321,6 +454,12 @@ mlfi_conn(SMFICTX *ctx,char *hostname,_SOCK_ADDR *hostaddr)
 	if (priv->c.peerstr == NULL)
 		priv->c.peerstr=(char*)malloc(strlen(buf)+1);
 	if (priv->c.peerstr) strcpy(priv->c.peerstr,buf);
+	mlfi_macros2vp(ctx,conn_names,priv->m.vp,&(priv->c.svmacros));
+	if ((myfqdn=vp_get_str(priv->m.vp,"j"))) {
+		if ((priv->c.myfqdn=(char*)malloc(strlen(myfqdn)+1)))
+			strcpy(priv->c.myfqdn,myfqdn);
+		vp_set_str(priv->m.vp,"myfqdn",myfqdn);
+	}
 	data.beg=buf;
 	data.end=buf+strlen(buf);
 	data.atr=(void *)&(priv->m.matr);
@@ -343,6 +482,7 @@ mlfi_helo(SMFICTX *ctx,char *helohost)
 		priv->c.helostr=(char*)malloc(strlen(helohost)+1);
 	}
 	if (priv->c.helostr) strcpy(priv->c.helostr,helohost);
+	mlfi_macros2vp(ctx,helo_names,priv->m.vp,&(priv->c.svmacros));
 	data.beg=helohost;
 	data.end=helohost+strlen(helohost);
 	data.atr=(void *)&(priv->m.matr);
@@ -356,6 +496,7 @@ mlfi_envfrom(SMFICTX *ctx,char **argv)
 	slab_t data;
 	int rc;
 	mlfiPriv_t *priv = mlfi_priv(ctx);
+	char *spoolid;
 
 	DPRINT(("mlfi_envfrom: %s\n",argv[0]));
 	if (priv == NULL) return SMFIS_TEMPFAIL;
@@ -381,9 +522,16 @@ mlfi_envfrom(SMFICTX *ctx,char **argv)
 		}
 	}
 	if (priv->m.freeze) return SMFIS_CONTINUE;
+	mlfi_macros2vp(ctx,envfrom_names,priv->m.vp,NULL);
+	if ((spoolid=vp_get_str(priv->m.vp,"i")))
+		vp_set_str(priv->m.vp,"spoolid",spoolid);
 	data.beg=argv[0];
 	data.end=argv[0]+strlen(argv[0]);
 	data.atr=(void *)&(priv->m.matr);
+	if ((*data.beg == '<') && (*(data.end-1) == '>')) {
+		data.beg++;
+		data.end--;
+	}
 	rc=scandata("envfrom",0,data,priv->m.vp);
 	return mlfi_termmode(ctx,rc);
 }
@@ -399,9 +547,14 @@ mlfi_envrcpt(SMFICTX *ctx,char **argv)
 	if (priv == NULL) return SMFIS_TEMPFAIL;
 	if (priv->c.local) return SMFIS_CONTINUE;
 	if (priv->m.freeze) return SMFIS_CONTINUE;
+	mlfi_macros2vp(ctx,envrcpt_names,priv->m.vp,NULL);
 	data.beg=argv[0];
 	data.end=argv[0]+strlen(argv[0]);
 	data.atr=(void *)&(priv->m.matr);
+	if ((*data.beg == '<') && (*(data.end-1) == '>')) {
+		data.beg++;
+		data.end--;
+	}
 	rc=scandata("envrcpt",0,data,priv->m.vp);
 	return mlfi_termmode(ctx,rc);
 }
@@ -495,6 +648,7 @@ mlfi_eom(SMFICTX *ctx)
 	static char *nul="nul";
 	slab_t data;
 	strl_t *xhdr;
+	int xpos=0;
 
 	DPRINT(("mlfi_eom\n"));
 	if (priv == NULL) return SMFIS_TEMPFAIL;
@@ -511,18 +665,21 @@ mlfi_eom(SMFICTX *ctx)
 
 		DPRINT(("add header: %s\n",xhdr->text));
 		p=strchr(xhdr->text,':');
-		if (p) {
-			ERRLOG((LOG_INFO,"Adding header \"%s\"",xhdr->text));
-		} else {
+		if (!p) {
 			ERRLOG((LOG_ERR,"bad extra header \"%s\"",xhdr->text));
 			continue;
 		}
+#if 0
+		else {
+			ERRLOG((LOG_INFO,"Adding header \"%s\"",xhdr->text));
+		}
+#endif
 		*p++='\0'; /* damage it, but it'll be destroyed soon anyway */
 		while ((*p == ' ') || (*p == '\t')) p++;
-		if (smfi_addheader(ctx,xhdr->text,p)) {
+		if (smfi_insheader(ctx,xpos,xhdr->text,p)) {
 			ERRLOG((LOG_ERR,"error adding header \"%s\" -> \"%s\"",
 				xhdr->text,p));
-		}
+		} else xpos++;
 	}
 
 #ifdef SMFIF_QUARANTINE
@@ -697,6 +854,10 @@ main(int argc, char *argv[])
 		}
 	}
 
+	if (strncmp(miltersock,"local:",6) == 0) {
+		(void)unlink(miltersock+6);
+	}
+
 	if (smfi_setconn(miltersock) == MI_FAILURE) {
 		ERRLOG((LOG_ERR,"smfi_setconn failed"));
 		exit(1);
