@@ -1,5 +1,5 @@
 /*
-	$Id: daemon.h 266 2005-04-17 13:37:59Z crosser $
+	$Id: daemon.h 421 2005-07-28 13:32:33Z crosser $
 
 	$Log: daemon.h,v $
 	Revision 1.7  2003/09/13 18:35:03  crosser
@@ -51,8 +51,16 @@ processfile(char *fn,varpool_t vp);
 extern int
 mainloop(int ctlsock,int argc, char *argv[]);
 
+extern void
+taspoolid(char *buf,time_t mtime,long inodenum);
+
+extern void
+getmyfqdn(char *name,size_t len,char *confdomain);
+
 extern int runchild;
 extern int stayalive;
 extern int isdaemon;
 
+extern char myfqdn[];
+
 #endif
