@@ -1,5 +1,5 @@
 /*
-	$Id: rtconfig.h 297 2005-05-13 17:44:56Z crosser $
+	$Id: rtconfig.h 421 2005-07-28 13:32:33Z crosser $
 */
 
 /*
@@ -23,6 +23,8 @@ typedef struct _gcfg {
 	char *pidfile;
 	char *modconfdir;
 	char *miltersock;
+	char *myfqdn;
+	char *mydomain;
 } gcfg_t;
 extern gcfg_t gcfg;
 
