@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 
-# $Id$
+# $Id: smtpsend.pl 397 2005-06-29 10:16:37Z crosser $
 
 # script for testing smscanner.  Submit messages to a host.
 
@@ -11,9 +11,13 @@ my $fn=$ARGV[0];
 
 print "sending $fn\n";
 
-my $sender = new Mail::Sender({smtp => 'pccross.average.org',
+my $sender = new Mail::Sender({smtp => 'www.average.org',
 				from => 'crosser@rol.ru',
-				debug => '/dev/tty'});
+				debug => '/dev/tty',
+				auth => 'PLAIN',
+				authid => 'testuser'.
+				authpwd => 'testpass',
+				});
 unless (ref $sender) {
 	print "new rc=$sender: ",$Mail::Sender::Error,"\n";
 	exit(1);
